/*
 * Copyright (C) 2011-2014 Gary Kramlich <grim@reaperworld.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 */

#include "gplugin-lua-plugin.h"

#include <lua.h>

/******************************************************************************
 * Typedefs
 *****************************************************************************/
typedef struct {
	lua_State *L;

	gchar *filename;
	GPluginLoader *loader;
	GPluginPluginInfo *info;
	GPluginPluginState state;
} GPluginLuaPluginPrivate;

/******************************************************************************
 * Enums
 *****************************************************************************/
enum {
	PROP_ZERO,
	PROP_LUA_STATE,
	N_PROPERTIES,
	/* overrides */
	PROP_FILENAME = N_PROPERTIES,
	PROP_LOADER,
	PROP_INFO,
	PROP_STATE
};
static GParamSpec *properties[N_PROPERTIES] = {NULL,};

/* I hate forward declarations... */
static void gplugin_lua_plugin_iface_init(GPluginPluginInterface *iface);

G_DEFINE_DYNAMIC_TYPE_EXTENDED(
	GPluginLuaPlugin,
	gplugin_lua_plugin,
	G_TYPE_OBJECT,
	0,
	G_ADD_PRIVATE_DYNAMIC(GPluginLuaPlugin)
	G_IMPLEMENT_INTERFACE(GPLUGIN_TYPE_PLUGIN, gplugin_lua_plugin_iface_init)
);

/******************************************************************************
 * GPluginPlugin Implementation
 *****************************************************************************/
static void
gplugin_lua_plugin_iface_init(GPluginPluginInterface *iface) {
}

/******************************************************************************
 * Object Stuff
 *****************************************************************************/
static void
gplugin_lua_plugin_get_property(GObject *obj, guint param_id, GValue *value,
                                   GParamSpec *pspec)
{
	GPluginLuaPlugin *plugin = GPLUGIN_LUA_PLUGIN(obj);
	GPluginLuaPluginPrivate *priv = gplugin_lua_plugin_get_instance_private(plugin);

	switch(param_id) {
		case PROP_LUA_STATE:
			g_value_set_pointer(value, priv->L);
			break;

		/* overrides */
		case PROP_FILENAME:
			g_value_set_string(value, priv->filename);
			break;
		case PROP_LOADER:
			g_value_set_object(value, priv->loader);
			break;
		case PROP_INFO:
			g_value_set_object(value, priv->info);
			break;
		case PROP_STATE:
			g_value_set_enum(value, priv->state);
			break;

		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(obj, param_id, pspec);
			break;
	}
}

static void
gplugin_lua_plugin_set_property(GObject *obj, guint param_id,
                                   const GValue *value, GParamSpec *pspec)
{
	GPluginLuaPlugin *plugin = GPLUGIN_LUA_PLUGIN(obj);
	GPluginLuaPluginPrivate *priv = gplugin_lua_plugin_get_instance_private(plugin);

	switch(param_id) {
		case PROP_LUA_STATE:
			priv->L = g_value_get_pointer(value);
			break;

		/* overrides */
		case PROP_FILENAME:
			priv->filename = g_strdup(g_value_get_string(value));
			break;
		case PROP_LOADER:
			priv->loader = g_object_ref(g_value_get_object(value));
			break;
		case PROP_INFO:
			priv->info = g_object_ref(g_value_get_object(value));
			break;
		case PROP_STATE:
			priv->state = g_value_get_enum(value);
			break;

		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(obj, param_id, pspec);
			break;
	}
}

static void
gplugin_lua_plugin_finalize(GObject *obj) {
	GPluginLuaPluginPrivate *priv = gplugin_lua_plugin_get_instance_private(GPLUGIN_LUA_PLUGIN(obj));

	if(priv->L)
		lua_close(priv->L);

	g_free(priv->filename);
	g_object_unref(G_OBJECT(priv->loader));
	g_object_unref(G_OBJECT(priv->info));

	G_OBJECT_CLASS(gplugin_lua_plugin_parent_class)->finalize(obj);
}

static void
gplugin_lua_plugin_init(GPluginLuaPlugin *plugin) {
}

static void
gplugin_lua_plugin_class_finalize(GPluginLuaPluginClass *klass) {
}

static void
gplugin_lua_plugin_class_init(GPluginLuaPluginClass *klass) {
	GObjectClass *obj_class = G_OBJECT_CLASS(klass);

	obj_class->get_property = gplugin_lua_plugin_get_property;
	obj_class->set_property = gplugin_lua_plugin_set_property;
	obj_class->finalize = gplugin_lua_plugin_finalize;

	properties[PROP_LUA_STATE] = g_param_spec_pointer(
		"lua-state", "lua-state",
		"The lua state for the plugin",
		G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY
	);

	g_object_class_install_properties(obj_class, N_PROPERTIES, properties);

	/* add our overrides */
	g_object_class_override_property(obj_class, PROP_FILENAME, "filename");
	g_object_class_override_property(obj_class, PROP_LOADER, "loader");
	g_object_class_override_property(obj_class, PROP_INFO, "info");
	g_object_class_override_property(obj_class, PROP_STATE, "state");
}

/******************************************************************************
 * API
 *****************************************************************************/
void
gplugin_lua_plugin_register(GPluginNativePlugin *native) {
	gplugin_lua_plugin_register_type(G_TYPE_MODULE(native));
}

lua_State *
gplugin_lua_plugin_get_state(GPluginLuaPlugin *plugin) {
	GPluginLuaPluginPrivate *priv = NULL;

	g_return_val_if_fail(GPLUGIN_IS_LUA_PLUGIN(plugin), NULL);

	priv = gplugin_lua_plugin_get_instance_private(plugin);

	return priv->L;
}
